/*
     KMiniDLNA Client
     https://sourceforge.net/projects/kminidlna/

     Copyright (C) 2012  Tomáš Poledný saljacky(at)gmail(dot)com

     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "kminidlna-client.h"
#include "mediafolderswidget.h"

#include <QtGui/QLabel>
#include <QtGui/QAction>
#include <QGridLayout>
#include <KCmdLineArgs>
#include <KDE/KLocalizedString>
#include <KStandardAction>
#include <KApplication>
#include <KMessageBox>
#include <KDialog>
#include <QColor>
#include <QTimer>
#include <QDomDocument>

KMiniDLNAClient::KMiniDLNAClient(): dialogShowed(false) {
    m_cli = new NetworkClient(this);
    initGUI();


    loadSettings();
    connect(m_cli, SIGNAL(stateRecieved(QString)), this, SLOT(onStateRecieved(QString)));
    connect(m_cli, SIGNAL(versionRecieved(QString)), this, SLOT(onVersionRecieved(QString)));
    connect(m_cli, SIGNAL(noConnection(int)), this, SLOT(onNoConnection(int)));
    connect(m_cli, SIGNAL(mediaFoldersLoaded(QString)), this, SLOT(onMediaFoldersLoaded(QString)));
    m_cli->reloadAll();

}

KMiniDLNAClient::~KMiniDLNAClient() {

}

void KMiniDLNAClient::initGUI() {
    m_mw = new QWidget(this);
    setCentralWidget(m_mw);
    QGridLayout* l = new QGridLayout(m_mw);
    m_mw->setLayout(l);

    setCaption(i18n("KMiniDLNA Client"));
    m_ledState = new KLed(Qt::gray, KLed::Off, KLed::Sunken, KLed::Circular, m_mw);

    m_state = new QLabel(i18n("Not loaded"), m_mw);

    l->addWidget(m_ledState, 0, 0);
    l->addWidget(m_state, 0, 1);

    l->addWidget(new QLabel(i18n("Host:"), m_mw), 1, 0);
    l->addWidget(new QLabel(i18n("Login:"), m_mw), 2, 0);
    l->addWidget(new QLabel(i18n("Password:"), m_mw), 3, 0);
    l->addWidget(new QLabel(i18n("Last update:"), m_mw), 5, 0);
    l->addWidget(new QLabel(i18n("Interface version:"), m_mw), 6, 0);

    m_host = new QLineEdit(m_mw);
    connect(m_host, SIGNAL(editingFinished()), this, SLOT(onHostEdited()));
    m_login = new QLineEdit(m_mw);
    connect(m_login, SIGNAL(editingFinished()), this, SLOT(onLoginEdited()));
    m_password = new QLineEdit(m_mw);
    connect(m_password, SIGNAL(editingFinished()), this, SLOT(onPasswordEdited()));
    m_password->setEchoMode(QLineEdit::Password);

    l->addWidget(m_host, 1, 1);
    l->addWidget(m_login, 2, 1);
    l->addWidget(m_password, 3, 1);

    m_remPass = new QCheckBox(i18n("Remember password"), m_mw);
    connect(m_remPass, SIGNAL(stateChanged(int)), SLOT(onRememeberPassword(int)));
    l->addWidget(m_remPass, 4, 1, 1, 2, Qt::AlignRight);

    m_lastUpdate = new QLabel(i18n("Not loaded"), m_mw);
    m_version = new QLabel(i18n("Not loaded"), m_mw);

    l->addWidget(m_lastUpdate, 5, 1);
    l->addWidget(m_version, 6, 1);

    //Buttons
    m_reload = new QPushButton(i18n("Reload state"), m_mw);
    connect(m_reload, SIGNAL(clicked(bool)), m_cli, SLOT(reloadAll()));

    m_folders = new QPushButton(i18n("Show folders"), m_mw);
    connect(m_folders, SIGNAL(clicked(bool)), this, SLOT(onFolders(bool)));

    m_start = new QPushButton(i18n("Start"), m_mw);
    connect(m_start, SIGNAL(clicked(bool)), this, SLOT(onStart()));

    l->addWidget(m_reload, 7, 0, 1, 2);
    l->addWidget(m_folders, 8, 0, 1, 2);
    l->addWidget(m_start, 9, 0, 1, 2);

//     connect(this, SIGNAL(destroyed(QObject*)), this, SLOT(quiteApp()));
    //menu
    menu = new KMenuBar(this);
    setMenuBar(menu);
    mTool = new KMenu(i18n("Tools"), menu);

    mTool->addAction((QAction*) KStandardAction::quit(this, SLOT(quiteApp()), this));

    menu->addMenu(mTool);
    aboutMenu = new KHelpMenu(menu, KCmdLineArgs::aboutData());
    mAbout = aboutMenu->menu();
    menu->addMenu(aboutMenu->menu());
}

void KMiniDLNAClient::quiteApp() {
    m_cli->saveSettings();
    kapp->quit();
}

void KMiniDLNAClient::loadSettings() {
    m_cli->loadSettings();
    m_host->setText(m_cli->host());
    m_login->setText(m_cli->login());
    m_password->setText(m_cli->password());
    m_remPass->setChecked(m_cli->rememberPassword());
}

void KMiniDLNAClient::onHostEdited() {
    m_cli->setHost(m_host->text());
}

void KMiniDLNAClient::onLoginEdited() {
    m_cli->setLogin(m_login->text());
}

void KMiniDLNAClient::onPasswordEdited() {
    m_cli->setPassword(m_password->text());
}

void KMiniDLNAClient::onRememeberPassword(int) {
    m_cli->setRememberPassword(m_remPass->isChecked());
}

void KMiniDLNAClient::onNoConnection(int rep) {
    lastUpdate();
    QNetworkReply::NetworkError err = (QNetworkReply::NetworkError) rep;
    switch(err) {
    case QNetworkReply::HostNotFoundError:
    case QNetworkReply::ConnectionRefusedError:
        if(!dialogShowed) {
            dialogShowed = true;
            KMessageBox::error(this, i18n("Problem with host. Please check host address."), i18n("Host problem"));
            dialogShowed = false;
        }
        break;
    case QNetworkReply::ContentAccessDenied:
    case QNetworkReply::AuthenticationRequiredError:
        if(!dialogShowed) {
            dialogShowed = true;
            KMessageBox::error(this, i18n("Problem with login or password. Please check login and password."), i18n("Wrong login or password"));
            dialogShowed = false;
        }
        break;
    case QNetworkReply::UnknownNetworkError:
//         KMessageBox::error(this, i18n("Unknown network error."), i18n("Unknown network error"));
        break;
    default:
        break;
    }

}

void KMiniDLNAClient::onStateRecieved(QString state) {
    lastUpdate();
    m_state->setText(state);
    if(state == "running") {
        m_start->setText(i18n("Stop"));
        m_ledState->setColor(Qt::green);
        m_ledState->setState(KLed::On);
    } else if(state == "notrunning") {
        m_start->setText(i18n("Start"));
        m_ledState->setColor(Qt::gray);
        m_ledState->setState(KLed::Off);
    }
}

void KMiniDLNAClient::onVersionRecieved(QString version) {
    lastUpdate();
    m_version->setText(version);
}


void KMiniDLNAClient::onStart() {
    m_cli->start();
    QTimer::singleShot(WAIT, m_cli, SLOT(reloadAll()));
}

void KMiniDLNAClient::onMediaFoldersLoaded(QString xmlDoc) {
    lastUpdate();
    m_model.clear();
    m_model.setColumnCount(2);
    QStringList headerLabels;
    headerLabels << i18n("Folder") << i18n("Media type");
    m_model.setHorizontalHeaderLabels(headerLabels);
    QDomDocument doc;
    if(doc.setContent(xmlDoc)) {
        QDomElement root = doc.documentElement();
        if(root.tagName().toLower() == "mediafolders") {
            QDomElement folder = root.firstChildElement("mediafolder");
            while(!folder.isNull()) {
                QDomElement el = folder.firstChildElement("path");

                if(!el.isNull()) {
                    QStandardItem* path = new QStandardItem(el.text());
                    el = folder.firstChildElement("mediatype");
                    QStandardItem* type;
                    if(!el.isNull()) {
                        type = new QStandardItem(el.text());
                    } else {
                        type = new QStandardItem(QString());
                    }
                    QList<QStandardItem*> list;
                    list.append(path);
                    list.append(type);
                    m_model.appendRow(list);
                }
                folder = folder.nextSiblingElement("mediafolder");
            }

        }
    }
    emit modelLoaded();

}

QStandardItemModel* KMiniDLNAClient::mediaFolderModel() {
    return &m_model;
}



void KMiniDLNAClient::onFolders(bool) {
    m_cli->realoadMediaFolders();
    KDialog dlg;
    dlg.setMainWidget(new MediaFoldersWidget(mediaFolderModel(), &dlg));
    dlg.setCaption(i18n("Media folders"));
    dlg.setButtons(KDialog::Ok);
    dlg.exec();
}

void KMiniDLNAClient::lastUpdate() {
    QDateTime d = QDateTime::currentDateTime();
    m_lastUpdate->setText(d.toString());
}
